package org.prelle.user;

/**
 * AccessMatrix.java
 *
 *
 * @author  Stefan Prelle
 * @version $Id$
 */

public class AccessMatrix implements AccessLevel {
    
    public final static AccessMatrix READ_ONLY    = new AccessMatrix(44444);
    public final static AccessMatrix R__          = new AccessMatrix(44444);
    public final static AccessMatrix READ_WRITE   = new AccessMatrix(66666);
    public final static AccessMatrix RW_          = new AccessMatrix(66666);
    public final static AccessMatrix WRITE_ONLY   = new AccessMatrix(22222);
    public final static AccessMatrix _W_          = new AccessMatrix(22222);
    public final static AccessMatrix EXECUTE_ONLY = new AccessMatrix(11111);
    public final static AccessMatrix ALL          = new AccessMatrix(77777);
    public final static AccessMatrix RWX          = new AccessMatrix(77777);
	    

    private boolean[][] matrix;

    //--------------------------------------------------
    public AccessMatrix() {
	matrix = new boolean[5][3];
    }

    //--------------------------------------------------
    public AccessMatrix(boolean[][] matr) {
	matrix = matr;
    }

    //--------------------------------------------------
    public AccessMatrix(int mask) {
	matrix = new boolean[5][3];
	int tmp = mask;
	for (int i=4; i>=0; i--) {
	    int tmp2 = tmp%10;
	    matrix[i] = new boolean[]{(tmp2 & 4)>0 ,(tmp2 & 2)>0 ,(tmp2 & 1)>0};
	    tmp = tmp/10;
	}
    }

    //--------------------------------------------------
    public boolean hasAccessOfType(int type, int level) {
	return matrix[level][type];
    }

    //--------------------------------------------------
    public boolean hasReadAccess(int level) {
	return matrix[level][READ];
    }

    //--------------------------------------------------
    public boolean hasWriteAccess(int level) {
	return matrix[level][WRITE];
    }

    //--------------------------------------------------
    public boolean hasExecuteAccess(int level) {
	return matrix[level][EXECUTE];
    }

    //--------------------------------------------------
    public void setAccessType(int type, int level, boolean hasAccess) {
	matrix[level][type] = hasAccess;
    }

    //--------------------------------------------------
    /**
     * Ensure that independent of the access level only
     * this type of access is granted.
     */
    public void restrictAccessTypeTo(int type) {
	for (int i=0; i<matrix.length; i++) {
	    for (int j=0; j<matrix[i].length; j++) {
		if (j==type)
		    matrix[i][j]= true;
		else
		    matrix[i][j]= false;
	    }
	}
    }

    //--------------------------------------------------
    public String dump() {
	StringBuffer buf = new StringBuffer();
	buf.append("         rwx\n");
	for (int l=0; l<5; l++) {
	    buf.append("Level "+l+" : ");
	    buf.append(matrix[l][READ]?"r":"-");
	    buf.append(matrix[l][WRITE]?"w":"-");
	    buf.append(matrix[l][EXECUTE]?"x":"-");
	    buf.append("\n");
	}
	return buf.toString();
    }

    //--------------------------------------------------
    public String toString() {
	StringBuffer buf = new StringBuffer();
	buf.append("[");
	for (int l=0; l<5; l++) {
	    buf.append("L"+l+":");
	    buf.append(matrix[l][READ]?"r":"-");
	    buf.append(matrix[l][WRITE]?"w":"-");
	    buf.append(matrix[l][EXECUTE]?"x":"-");
	    if (l<4)
		buf.append(", ");
	}
	buf.append("]");
	return buf.toString();
    }
    
    
}// AccessMatrix
